﻿using System.Collections.Generic;
using System.Data.Entity.Migrations;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using lecture4.dto;
using lecture4.repository.Entities;

namespace lecture4.repository.Repositories
{
    public class CommentRepository
    {
        public Task<List<Comment>> GetByIssueAsync(int issueId)
        {
            using (var context = DataContext.Get())
            {
                var result = context.Comments
                    .Where(c => c.IssueId == issueId)
                    .ToList()
                    .Select(Mapper.Map<Comment>)
                    .ToList();

                return Task.FromResult(result);
            }
        }

        public Task<List<Comment>> GetByAuthorAsync(string author)
        {
            using (var context = DataContext.Get())
            {
                var result = context.Comments
                    .Where(c => c.Author.Equals(author))
                    .ToList()
                    .Select(Mapper.Map<Comment>)
                    .ToList();

                return Task.FromResult(result);
            }
        }

        public Task<Comment> AddOrUpdateAsync(Comment comment)
        {
            using (var context = DataContext.Get())
            {
                var entity = Mapper.Map<CommentEntity>(comment);

                context.Comments.AddOrUpdate(entity);
                context.SaveChanges();

                return Task.FromResult(Mapper.Map<Comment>(entity));
            }
        }

        public static void CreateMap()
        {
            Mapper.CreateMap<Comment, CommentEntity>()
                .ForMember(e => e.Id, conf => conf.MapFrom(c => c.Id))
                .ForMember(e => e.Author, conf => conf.MapFrom(c => c.Author))
                .ForMember(e => e.IssueId, conf => conf.MapFrom(c => c.IssueId))
                .ForMember(e => e.Text, conf => conf.MapFrom(c => c.Text))
                .ForMember(e => e.Issue, conf => conf.Ignore());

            Mapper.CreateMap<CommentEntity, Comment>()
                .ForMember(c => c.Id, conf => conf.MapFrom(e => e.Id))
                .ForMember(c => c.Author, conf => conf.MapFrom(e => e.Author))
                .ForMember(c => c.IssueId, conf => conf.MapFrom(e => e.IssueId))
                .ForMember(c => c.Text, conf => conf.MapFrom(e => e.Text));
        }
    }
}
