﻿using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Migrations;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using lecture4.dto;
using lecture4.repository.Entities;

namespace lecture4.repository.Repositories
{
    public class IssueRepository
    {
        public Task<List<Issue>> GetAllAsync()
        {
            using (var context = DataContext.Get())
            {
                var result = context.Issues
                    .Include(e => e.Comments)
                    .ToList()
                    .Select(Mapper.Map<Issue>)
                    .ToList();

                return Task.FromResult(result);
            }
        }

        public Task<Issue> GetOneAsync(int issueId)
        {
            using (var context = DataContext.Get())
            {
                var entity = context.Issues
                    .Include(e => e.Comments)
                    .FirstOrDefault(e => e.Id == issueId);

                return Task.FromResult(Mapper.Map<Issue>(entity));
            }
        }

        public async Task<Issue> AddOrUpdateAsync(Issue issue)
        {
            using (var context = DataContext.Get())
            {
                var entity = Mapper.Map<IssueEntity>(issue);

                context.Issues.AddOrUpdate(entity);
                await context.SaveChangesAsync();

                return Mapper.Map<Issue>(entity);
            }
        }

        public async Task<bool> DeleteAsync(int issueId)
        {
            using (var context = DataContext.Get())
            {
                var existing = context.Issues.FirstOrDefault(i => i.Id == issueId);

                if (existing == null)
                    return false;

                context.Issues.Remove(existing);
                await context.SaveChangesAsync();

                return true;
            }
        }

        public static void CreateMap()
        {
            Mapper.CreateMap<Issue, IssueEntity>()
                .ForMember(e => e.Id, conf => conf.MapFrom(i => i.Id))
                .ForMember(e => e.Owner, conf => conf.MapFrom(i => i.Owner))
                .ForMember(e => e.Summary, conf => conf.MapFrom(i => i.Summary))
                .ForMember(e => e.Type, conf => conf.MapFrom(i => i.Type))
                .ForMember(e => e.Comments, conf => conf.MapFrom(i => i.Comments));

            Mapper.CreateMap<IssueEntity, Issue>()
                .ForMember(i => i.Id, conf => conf.MapFrom(e => e.Id))
                .ForMember(i => i.Owner, conf => conf.MapFrom(e => e.Owner))
                .ForMember(i => i.Summary, conf => conf.MapFrom(e => e.Summary))
                .ForMember(i => i.Type, conf => conf.MapFrom(e => e.Type))
                .ForMember(i => i.Comments, conf => conf.ResolveUsing(e => e.Comments.ToList()));
        }
    }
}
