﻿using System.Linq;
using System.Net;
using System.Threading.Tasks;
using System.Web.Http;
using lecture4.dto;
using lecture4.repository.Repositories;

namespace lecture4.Controllers
{
    [RoutePrefix("issues")]
    public class IssueController : ApiController
    {
        private readonly IssueRepository _issueRepository;
        private readonly CommentRepository _commentRepository;

        public IssueController(IssueRepository issueRepository, CommentRepository commentRepository)
        {
            _issueRepository = issueRepository;
            _commentRepository = commentRepository;
        }
        
        // TODO: Get all issues
        // TODO: Get One issue
        // TODO: Create issue
        // TODO: Update issue
        // TODO: Delete issue

        // TODO: Comment on issue
        // TODO: Get user comments + paging
        
        // TODO: Top 5 comments

        [HttpGet]
        [Route("")]
        public async Task<IHttpActionResult> GetAll()
        {
            return Ok(await _issueRepository.GetAllAsync());
        }

        [HttpGet]
        [Route("{id:int}")]
        public async Task<IHttpActionResult> GetOne(int id)
        {
            var result = await _issueRepository.GetOneAsync(id);

            if (result == null)
                return NotFound();

            return Ok(result);
        }

        [HttpPost]
        [Route("")]
        public async Task<IHttpActionResult> Insert([FromBody]Issue issue)
        {
            if (!ModelState.IsValid)
                return BadRequest();

            if (issue.Id != 0)
                return BadRequest();

            var result = await _issueRepository.AddOrUpdateAsync(issue);

            return Created(Request.RequestUri + "/" + result.Id, result);
        }

        [HttpPut]
        [Route("{id:int}")]
        public async Task<IHttpActionResult> Update(int id, [FromBody] Issue issue)
        {
            if (!ModelState.IsValid)
                return BadRequest();

            if (issue.Id != id)
                return BadRequest();

            var existing = await _issueRepository.GetOneAsync(id);

            if (existing == null)
                return NotFound();

            var result = await _issueRepository.AddOrUpdateAsync(issue);

            return Ok(result);

        }

        [HttpDelete]
        [Route("{id:int}")]
        public async Task<IHttpActionResult> Delete(int id)
        {
            var deleted = await _issueRepository.DeleteAsync(id);

            if (deleted)
            {
                return StatusCode(HttpStatusCode.NoContent);
            }

            return NotFound();
        }


        [HttpGet]
        [Route("{id:int}/comments")]
        public async Task<IHttpActionResult> GetComments(int id)
        {
            var issue = await _issueRepository.GetOneAsync(id);

            if (issue == null)
                return NotFound();

            return Ok(issue.Comments);
        }

        [HttpPost]
        [Route("{id:int}/comments")]
        public async Task<IHttpActionResult> CommentOnIssue(int id, [FromBody] Comment comment)
        {
            if (comment.Id != 0)
                return BadRequest();

            if (comment.IssueId != id)
                return BadRequest();

            var issue = await _issueRepository.GetOneAsync(id);

            if (issue == null)
                return BadRequest();

            if (issue.Comments.Any(c => c.Id == comment.Id))
                return BadRequest();

            var result = await _commentRepository.AddOrUpdateAsync(comment);

            return Created(Request.RequestUri + "/" + result.Id, result);
        }

        [HttpGet]
        [Route("{id:int}/topcomments")]
        public async Task<IHttpActionResult> GetTopComments(int id)
        {
            var issue = await _issueRepository.GetOneAsync(id);

            if (issue == null)
                return NotFound();

            return Ok(issue.Comments.Take(5));
        }
    }
}